@@ -489,6 +489,7 @@ class DataConstantsTransformerForm(forms.ModelForm):
                             widget=forms.HiddenInput(attrs={'class': 'form-control'}))
     value = forms.CharField(required=False,
                             widget=forms.TextInput(attrs={'class': 'form-control'}))
+    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformerTypes.objects.all(), widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
     class Meta:
         model = DataTransformerConstants
@@ -500,6 +501,7 @@ class HardwareTransformerForm(forms.ModelForm):
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}))
     index = forms.CharField(required=False,
                             widget=forms.HiddenInput(attrs={'class': 'form-control'}))
+    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformerTypes.objects.all(), widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
     class Meta:
         model = HardwareDataTransformer
@@ -511,6 +513,8 @@ class HardwareStatsTransformerForm(forms.ModelForm):
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}))
     index = forms.CharField(required=False,
                             widget=forms.HiddenInput(attrs={'class': 'form-control'}))
+    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformerTypes.objects.all(), widget=forms.HiddenInput(attrs={'class': 'form-control'}))
+
 
     class Meta:
         model = HardwareStatsDataTransformer
@@ -694,6 +698,7 @@ class DataTransformRoot(forms.ModelForm):
                                                                      can_delete_extra=True
                                                                      )
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
+                    instance=self.instance,
                     queryset=self.instance.get_children(),
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
@@ -713,7 +718,8 @@ class DataTransformRoot(forms.ModelForm):
             print('exception')
 
         self.nested_constants_form = DataConstantsTransformerInlineFormset(
-            queryset=self.instance.datatransformerconstants_set,
+            instance=self.instance,
+            queryset=self.instance.get_children(),
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
             initial=[{'data_transformer': self.instance, 'id': str(uuid.uuid4())}]
@@ -721,7 +727,8 @@ class DataTransformRoot(forms.ModelForm):
         )
 
         self.nested_hardware_outputs_form = HardwareTransformerInlineFormset(
-            queryset=self.instance.hardwaredatatransformer_set,
+            instance=self.instance,
+            queryset=self.instance.get_children(),
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
             initial=[{'data_transformer': self.instance, 'id': str(uuid.uuid4())}]
@@ -729,7 +736,8 @@ class DataTransformRoot(forms.ModelForm):
         )
 
         self.nested_hardware_stats_form = HardwareStatsTransformerInlineFormset(
-            queryset=self.instance.hardwarestatsdatatransformer_set,
+            instance=self.instance,
+            queryset=self.instance.get_children(),
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
             initial=[{'data_transformer': self.instance, 'id': str(uuid.uuid4())}]
@@ -761,12 +769,15 @@ class DataTransformRoot(forms.ModelForm):
         root_node = super(DataTransformRoot, self).save(commit)
         if hasattr(self,
                    'nested_constants_form') and self.nested_constants_form is not None:
+            self.nested_constants_form.instance.refresh_from_db()
             if self.nested_constants_form.is_valid():
                 self.nested_constants_form.save(commit)
             else:
                 print(self.nested_constants_form.errors)
         if hasattr(self,
                    'nested_hardware_outputs_form') and self.nested_hardware_outputs_form is not None:
+            self.nested_hardware_outputs_form.instance.refresh_from_db()
+
             if self.nested_hardware_outputs_form.is_valid():
                 self.nested_hardware_outputs_form.save(commit)
             else:
@@ -779,6 +790,8 @@ class DataTransformRoot(forms.ModelForm):
                 print(self.nested_hardware_stats_form.errors)
         if hasattr(self,
                    'nested_data_transformer_form') and self.nested_data_transformer_form is not None:
+            self.nested_data_transformer_form.instance.refresh_from_db()
+
             if self.nested_data_transformer_form.is_valid():
                 self.nested_data_transformer_form.save(commit)
             else:
